import com.dalsemi.onewire.adapter.TINIInternalAdapter;
import com.dalsemi.onewire.OneWireException;
import com.dalsemi.onewire.utils.CRC16;

class WriteIP {
	static final int TARGET_FAMILY_ID         = 0x23;
	static final int READ_MEMORY_COMMAND      = 0xf0;
	static final int WRITE_SCRATCHPAD_COMMAND = 0x0f;
	static final int COPY_SCRATCHPAD_COMMAND  = 0x55;
	static final int READ_SCRATCHPAD_COMMAND  = 0xaa;

	public static void main(String[] args) {
		TINIInternalAdapter adapter = new TINIInternalAdapter();
		boolean foundIt = false;

		try {
			long deviceAddress;

			adapter.beginExclusive(true);
			if (adapter.findFirstDevice()) {
				// Test LSB (family id) against target
				deviceAddress = adapter.getAddressAsLong();
				if ((deviceAddress & 0xff) == TARGET_FAMILY_ID) {
					foundIt = true;
				}
				while (!foundIt && adapter.findNextDevice()) {
					deviceAddress = adapter.getAddressAsLong();
					if ((deviceAddress & 0xff) == TARGET_FAMILY_ID) {
						foundIt = true;
					}
				}
				if (foundIt) {
					byte[] command = new byte[4];
					command[0] = (byte) READ_MEMORY_COMMAND;
					command[1] = 0;
					command[2] = 0;
					adapter.select(deviceAddress);
					adapter.dataBlock(command, 0, 3);

					byte[] IPdata = adapter.getBlock(32);

					for (int i = 0; i < 32; i++) {
						System.out.print(Integer.toHexString(IPdata[i] & 0xff));
					}
					System.out.println();

					int crc = CRC16.compute(IPdata);
                			System.out.println("original crc=" + Integer.toHexString(crc));

					byte[] config = new byte[32];
					config[0] = 29;	// length
					config[1] = 'T';
					config[2] = 'I';
					config[3] = 'N';
					config[4] = 'I';
					config[5] = (byte) 192;
					config[6] = (byte) 168;
					config[7] = (byte) 0;
					config[8] = (byte) 18;
					config[9] = (byte) 192;
					config[10] = (byte) 168;
					config[11] = (byte) 0;
					config[12] = (byte) 1;
					config[13] = 24;
					config[14] = 0;
					config[15] = 0;
					config[16] = 0;
					config[17] = 0;
					config[18] = 0;
					config[19] = 0;
					config[20] = 0;
					config[21] = 0;
					config[22] = 0;
					config[23] = 0;
					config[24] = 0;
					config[25] = 0;
					config[26] = (byte) 192;
					config[27] = (byte) 168;
					config[28] = (byte) 0;
					config[29] = (byte) 2;

					crc = ~CRC16.compute(config, 0, 30);
					config[30] = (byte) (crc & 0xff);
					config[31] = (byte) ((crc >> 8) & 0xff);

					for (int i = 0; i < 32; i++) {
						System.out.print(Integer.toHexString(config[i] & 0xff));
					}
					System.out.println();

					command[0] = (byte) WRITE_SCRATCHPAD_COMMAND;
					command[1] = 0;
					command[2] = 0;
					adapter.select(deviceAddress);
					adapter.dataBlock(command, 0, 3);
					adapter.dataBlock(config, 0, 32);

					command[0] = (byte) READ_SCRATCHPAD_COMMAND;
					adapter.select(deviceAddress);
					adapter.dataBlock(command, 0, 1);
					byte[] scratch = adapter.getBlock(3+32);

					for (int i = 0; i < 32+3; i++) {
						System.out.print(Integer.toHexString(scratch[i] & 0xff));
					}
					System.out.println();

					command[0] = (byte) COPY_SCRATCHPAD_COMMAND;
					command[1] = 0;
					command[2] = 0;
					command[3] = scratch[2];
					adapter.select(deviceAddress);
					adapter.dataBlock(command, 0, 4);

				} else {
					System.out.println("Device not found");
				}
			}
		} catch (OneWireException owe) {
			System.out.println(owe.getMessage());
		} finally {
			adapter.endExclusive();
		}
	}
}
